% Program demo_5_1
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Sampling rate conversion with M=L=2.
% 5th order IIR half-band filter
 close all, clear all
% Allpass branches
p = [0.4863i,-0.4863i,0.8453i,-0.8453i];  % Halfband filter poles
beta0 = (abs(p(1)))^2;    beta1 = (abs(p(3)))^2;  % Coefficients of allpass branches
[A0,f] = freqz([beta0,0,1],[1,0,beta0],512,2); [A1,f] = freqz([0,beta1,0,1],[1,0,beta1],512,2);
 
% Half-band filter frequency response
H = (A0 + A1)/2;  % Computing the IIR halfband filter frequency response
figure (1)
subplot(4,1,1), plot(f,20*log10(abs(H)))
ylabel('Gain, dB'), axis([0,1,-60,2])
text(0.65,-20,'Filter Characteristic')
 
% Generating the input signal
F = [0,0.1,0.46,1]; A=[0,1,0,0]; x1 = fir2(256,F,A);
x = x1 +  0.008*cos(2*pi*0.35*(0:256));% Original signal
X = fft(x); % Spectrum of the original signal
subplot(4,1,2), plot((0:127)/128,abs(X(1:128))), ylabel('|X|'), axis([0,1,0,1.2])
 
% Decimation-by-2
u0 = x(1:2:length(x));
u1 = [0,x(2:2:length(x)-1)];
y0 = filter([beta0,1],[1,beta0],u0);
y1 = filter([beta1,1],[1,beta1],u1);
ydec = (y0 + y1)/2;
Ydec = fft(ydec);
subplot(4,1,3), plot((0:63)/64,abs(Ydec(1:64))), ylabel('|Y_{dec}|'), axis([0,1,0,0.6])
 
%Interpolation
y0 = filter([beta0,1],[1,beta0],ydec);
y1 = filter([beta1,1],[1,beta1],ydec);
yy = [y0;y1];
yint = yy(:);
Yint = fft(yint);
subplot(4,1,4), plot((0:127)/128,abs(Yint(1:128)))
xlabel('\omega/\pi'), ylabel('|Y_{int}|'),  axis([0,1,0,1.2])
