% Example_5_2
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% IIR 5th band polyphase filter
clear all, close all
% IIR filter poles
z01 = -0.03247627480; z02 = -0.4519480048; z03 = -0.9477051753;
z11 = -0.08029157130; z12 = -0.5548998293;
z21 = -0.1417079348; z22 = -0.6883346404;
z31 = -0.2320513100; z32 = -0.7961481351;
z41 = -0.3532045984; z42 = -0.8755417392;
% Polyphase branches
a0 = conv([1,-z01],[1,-z02]); a0 = conv(a0,[1,-z03]);
a1 = conv([1,-z11],[1,-z12]);
a2 = conv([1,-z21],[1,-z22]);
a3 = conv([1,-z31],[1,-z32]);
a4 = conv([1,-z41],[1,-z42]);

A0=upsample(a0,5); A1=upsample(a1,5); A2=upsample(a2,5); A3=upsample(a3,5); A4=upsample(a4,5);
B0=fliplr(A0); B1=[0,fliplr(A1)]; B2=[0,0,fliplr(A2)]; B3=[0,0,0,fliplr(A3)]; B4=[0,0,0,0,fliplr(A4)]; 

% Computing the phase responses of polyphase branches
[PH0,f] = freqz(B0,A0,1024,2); [PH1,f] = freqz(B1,A1,1024,2); [PH2,f] = freqz(B2,A2,1024,2);
[PH3,f] = freqz(B3,A3,1024,2); [PH4,f] = freqz(B4,A4,1024,2);
figure (1)
plot(f,unwrap(angle(PH0)),f,unwrap(angle(PH1)),'b',f,unwrap(angle(PH2)),'b',f,unwrap(angle(PH3)),'b',...
f,unwrap(angle(PH4)),'b')
xlabel('Normalized frequency [\omega/\pi]')
ylabel('Phase [rad]')
% Overall IIR filter frequency response
H=PH0+PH1+PH2+PH3+PH4;
figure(2)
subplot(2,1,1), plot(f,20*log10(abs(H)/5),'LineWidth',2), grid
hold on
subplot(2,1,2), plot(f,unwrap(angle(H)),'LineWidth',2)
hold on

% Equivalent FIR filter
ap = 0.000015; as = 60; Fp = 0.8/5;      
dev = [(10^(ap/20)-1)/(10^(ap/20)+1),10^(-as/20),10^(-as/20),10^(-as/20)]; 
F = [Fp,2/5-Fp,2/5+Fp,4/5-Fp,4/5+Fp,1];   % Cutoff frequencies
A = [1,0,0,0];      % Desired amplitudes
[Nord,Fo,Ao,W] = firpmord(F,A,dev); % Estimating the filter order
b = firpm(126,Fo,Ao,W); % Computing the filter coefficients
[Hfir,f] = freqz(b,1,1024,2); % Computing the filter frequency response
figure (2)
subplot(2,1,1)
plot(f,20*log10(abs(Hfir))), ylabel('Magnitude [dB]'), axis([0,1,-80,2])
subplot(2,1,2), plot(f,unwrap(angle(Hfir)))
xlabel('Normalized frequency [\omega/\pi]'), ylabel('Phase [rad]'),axis([0,1,-50,0])
 
% Plot the passband details: IIR and FIR
figure (3)
subplot(2,1,1)
plot(f(1:164),20*log10(abs(H(1:164))/5),'LineWidth',2)
hold on
figure (3)
plot(f(1:164),20*log10(abs(Hfir(1:164)))),grid
xlabel('Normalized frequency [\omega/\pi]'), ylabel('Magnitude, dB')


% Generating the input signal {x[n]}
n = 0:510;
x = cos(2*pi*200*n/10000) + cos(2*pi*400*n/10000) + 0.7*cos(2*pi*3800*n/10000) + randn(size(n));
[X,F1] = freqz(x,1,1024,10000);

% IIR filter, structure verification: polyphase downsampling and filtering
x0 = x(1:5:length(x)); x1 = [0,x(5:5:length(x))]; x2 = [0,x(4:5:length(x))]; 
x3 = [0,x(3:5:length(x))]; x4 = [0,x(2:5:length(x))];
y0 = filter(fliplr(a0)/5,a0,x0); y1 = filter(fliplr(a1)/5,a1,x1); y2 = filter(fliplr(a2)/5,a2,x2);
y3 = filter(fliplr(a3)/5,a3,x3); y4 = filter(fliplr(a4)/5,a4,x4);
ydec = y0 + y1 + y2 + y3 + y4; % Decimated signal
[Ydec,F2] = freqz(ydec,1,512,2000);

figure (4)
subplot(2,1,1), plot(F1,abs(X)), legend('Spectrum: input signal'), ylabel('|X(F)|'), axis([0,5000,0,300])
subplot(2,1,2),
plot(F2,abs(Ydec)), legend('Spectrum: decimated signal'), ylabel('|Y_d_e_c(F)|'), xlabel('Frequency [Hz]')

