% Example_5_5
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Sampling rate conversion with M=L=2.
% 5th order IIR half-band filter
close all, clear all
% Allpass branches
p=[0.4863i,-0.4863i,0.8453i,-0.8453i];  % Halfband filter poles
beta2HB=(abs(p(1)))^2;    beta3HB=(abs(p(3)))^2;  % Coefficients of allpass branches
[A0HB,w]=freqz([beta2HB,0,1],[1,0,beta2HB],1000); 
[A1HB,w]=freqz([0,beta3HB,0,1],[1,0,beta3HB],1000);

[A0HBu,w]=freqz(upsample([beta2HB,0,1],2),upsample([1,0,beta2HB],2),1000); 
[A1HBu,w]=freqz(upsample([0,beta3HB,0,1],2),upsample([1,0,beta3HB],2),1000);
H_HBu=(A0HBu+A1HBu)/2; 

[A0HBu4,w]=freqz(upsample([beta2HB,0,1],4),upsample([1,0,beta2HB],4),1000); 
[A1HBu4,w]=freqz(upsample([0,beta3HB,0,1],4),upsample([1,0,beta3HB],4),1000);
H_HBu4=(A0HBu4+A1HBu4)/2; 

% Half-band filter frequency response
H_HB=(A0HB+A1HB)/2; 
figure (1)
subplot(4,1,1), plot(w/pi,20*log10(abs(H_HB)))
%text(0.33,-10,'Filter characteristic')
ylabel('Gain, dB'), axis([0,1,-60,2]), grid

N=7
alpha=-(1/8+1/32);
%alpha=-0.1584;
f3db=acos(-alpha)/(2*pi)
%f3db=0.225;
ksi=(tan(2*pi*0.25/2)/tan(2*pi*f3db/2))^2
omegap=2*atan(1/sqrt(ksi));
fp=omegap/(2*pi)
[b,a,z,p,k] = halfbandiir(7,2*fp)
b=sort(abs(p));
betaHB=b(2:2:N).^2
alpha=-cos(2*pi*f3db);
alpha1=(1-sqrt(1-alpha^2))/alpha;

beta=(betaHB+alpha1^2)./(betaHB*alpha1^2+1);
a1=alpha1;
disp(' EMQF filter constants beta = ')
disp(beta')
p1 = [1 a1];
for ind = 2:2:(N-1)/2
  p1 = conv(p1,[1 alpha*(1+beta(ind)) beta(ind)]);
end
p2 = 1;
for ind = 1:2:(N-1)/2
  p2 = conv(p2,[1 alpha*(1+beta(ind)) beta(ind)]);
end

Hab=p1;
Hai=fliplr(Hab);
Hbb=p2;
Hbi=fliplr(Hbb);

[Ha,f]=freqz(Hab,Hai,1000,2);
[Hb,f]=freqz(Hbb,Hbi,1000,2);
H_EMQF=(Ha+Hb)/2;
subplot(4,1,2)
plot(f,20*log10(abs(H_EMQF)))
ylabel('Gain, dB'), axis([0,1,-60,2]), grid
Hab_u=upsample(Hab,4);
Hai_u=upsample(Hai,4);
Hbb_u=upsample(Hbb,4);
Hbi_u=upsample(Hbi,4);
[Ha_u,f]=freqz(Hab_u,Hai_u,1000,2);
[Hb_u,f]=freqz(Hbb_u,Hbi_u,1000,2);
H_EMQF_u=(Ha_u+Hb_u)/2;
H_SINGLE=H_HB.*H_HBu.*H_EMQF_u;
H_SINGLE_HB=H_HB.*H_HBu.*H_HBu4;
subplot(4,1,3:4)
plot(f,20*log10(abs(H_SINGLE)),f,20*log10(abs(H_SINGLE_HB)),'--b','LineWidth',2 )
grid
xlabel('\omega/\pi'), ylabel('Gain, dB')
axis([0,1,-100,5])

figure (2)
subplot(3,1,1)
plot(f(1:102),20*log10(abs(H_SINGLE(1:102))),f(1:102),20*log10(abs(H_SINGLE_HB(1:102))),'--b','LineWidth',2 )
grid
xlabel('\omega/\pi'), ylabel('Gain, dB')
axis([0,0.1,-3*0.001,0.5*0.001])