function [f,aaa]=example_5_6(int2,int3)
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Insert int2=5, int3=15
% Multirate filtering, 3 stage
close all, clear all
int2=5
int3=15
figure (1)
subplot(2,1,1)
[amp,f,beta,Ap,As,Fp,F3db]=emqf_fs(5,-(1-1/8+1/64),0.162);
amp_r=amp;
hold on
subplot(2,1,1)
[amp,f,beta,Ap,As,Fp,F3db]=emqf_fs_int(7,-0.5,0.24,int2);
f=2*f;
 amp_s=amp;
subplot(2,1,1)
[amp,f,beta,Ap,As,Fp,F3db]=emqf_fs_int(9,0,0.29,int3);
amp_g=amp;
ylabel('Gain, dB')
axis([0,1,-80,5])
subplot(2,1,2)
aaa=amp_r.*amp_s.*amp_g;
f=f;
plot(f,20*log10(abs(aaa)))
axis([0,1,-80,2])
xlabel('\omega/\pi')
ylabel('Gain, dB')
% -------------------------------------------------------------------------

function [amp,f,beta,Ap,As,Fp,F3db]=emqf_fs(N,alpha,Fs)
%
% emqf_fs	EMQF (Elliptic Minimal Q Factors) digital filter design.
%		          [amp,f,g,Ap,As,Fp,F3db]=emqf_1(N,rx0,Fs) designs an N'th order
%               lowpass digital EMQF filter with normalized stopband edge
%               frequency Fs. The sampling frequency is assumed to be F0=1.
%               The constant alfa is chosen by designer. For given Fp and Fs
%               (Fp and Fs the boundary frequencies of the specificatin),
%               the permited values alfa are in the range:
%
%   -(1-(tan(pi*Fa)^2)/(1+(tan(pi*Fa)^2)<alfa<-(1-(tan(pi*Fp)^2)/(1+(tan(pi*Fp)^2).
%
%	   The value of alfa approximates 
%              
%               (1-(tan(pi*Fa)(tan(pi*Fp))/(1+(tan(pi*Fa)(tan(pi*Fp))
% 
%               and is adjusted to be represented as a simple combination
%               of powers of two. 
%    
%     In the case of a half-band filter alfa=0         
%
%       emqf_fs returns:
%              * Amplitude response in vector amp, length 500. 
%              * Normalized frequencies (F0=1) in vector f, length 500. 
%              * Square pole magnitudes of the complex poles in vector beta. 
%              * passband ripple Ap (dB)
%              * minimal stopband attenuation As (dB)
%              * passband boundary frequency of EMQF filter Fp
%              * 3 dB attenuation frequency F3db
%
%

disp(' EMQF design')
disp('===============================')
disp([sprintf(' N     = %6u', N)])
disp([sprintf(' alpha = %15.9f', alpha)])
disp([sprintf(' F_s   = %15.9f', Fs)])

F3db = (1/pi)*atan(sqrt((1+alpha)/(1-alpha)));

neven = N-2*fix(N/2);
if neven < 0.5
   disp('------- error -----------------')
   disp('N should be odd number')
   disp('choose new N = 3, 5, 7, 9, 11, ...')
   disp('-------------------------------')
   H=0; w=0; zzd=0; ppd=0; Ap=0; As=0; Fp=0; F3db=0;omega=0;co=0;
   return
end

if F3db >= Fs
   disp([sprintf(' F_3dB = %15.9f', F3db)])
   disp('------- error -----------------')
   disp('Fs should be larger than F3dB')
   disp(['choose new Fs    > ' num2str(F3db) '  or'])
   disp(['choose new alpha < ' num2str(-cos(2*pi*Fs))])
   disp('-------------------------------')
   H=0; w=0; zzd=0; ppd=0; Ap=0; As=0;omega=0;co=0;
   return
end

Fp = (1/pi)*atan(((1+alpha)/(1-alpha))*(1/tan(pi*Fs)));
disp([sprintf(' F_p   = %15.9f', Fp)])
disp([sprintf(' F_3dB = %15.9f', F3db)])

xi = tan(pi*Fs)/tan(pi*Fp);
if xi >= sqrt(2)
   t = .5*(1-(1-1/xi^2)^(1/4))/(1+(1-1/xi^2)^(1/4));
   q = t+2*t^5+15*t^9+150*t^13;
else
   t = .5*(1-1/sqrt(xi))/(1+1/sqrt(xi));
   qp = t+2*t^5+15*t^9+150*t^13;
   q = exp(pi^2/log(qp));
end
L  = (1/4)*sqrt(1/q^N-1);
Ap = 10*log10(1+1/L);
As = 10*log10(1+L);
disp([sprintf(' A_p   =     %15.9g', Ap)])
disp([sprintf(' A_s   = %15.9f', As)])

%[zzd,ppd,co]=ellip(N,Ap,As,2*Fp);
a = xi;
b = 1; 
for ind = 1:(N-1)/2
  x = ellipj( ((2*ind-1)/N + 1)*ellipke(1/a^2), 1/a^2);
  kor = 2*sqrt(a)*tan(pi*Fp)/(1+a*tan(pi*Fp)^2);
  b(ind) = ((a+x^2) - kor*sqrt((1-x^2)*(a^2-x^2)) )/...
           ((a+x^2) + kor*sqrt((1-x^2)*(a^2-x^2)) );
end
a1 = -(1-tan(pi*F3db))/(1+tan(pi*F3db));
beta = sort(b);
disp([sprintf(' alpha1 = %15.9f', a1)])
disp(' beta = ')
disp(beta')

p1 = [1 a1];
for ind = 2:2:(N-1)/2
  p1 = conv(p1,[1 alpha*(1+beta(ind)) beta(ind)]);
end
p2 = 1;
for ind = 1:2:(N-1)/2
  p2 = conv(p2,[1 alpha*(1+beta(ind)) beta(ind)]);
end

Hab=p1;
Hai=fliplr(Hab);
Hbb=p2;
Hbi=fliplr(Hbb);

[Ha,w]=freqz(Hab,Hai,2000);
[Hb,w]=freqz(Hbb,Hbi,2000);
fia=unwrap(angle(Ha));
fib=unwrap(angle(Hb));
psi=(fia-fib)/2;
phase=fia+fib;
amp=cos(psi);
f=w/pi;
plot(f,20*log10(abs(amp)),'r');

% ---------------------------------------------------------------------------------

function [amp,f,beta,Ap,As,Fp,F3db]=emqf_fs_int(N,alpha,Fs,M)
% Interpolated EMQF filter, M interpolation factor
% emqf_fs_int	EMQF (Elliptic Minimal Q Factors) digital filter design.
%		          [amp,f,g,Ap,As,Fp,F3db]=emqf_1(N,rx0,Fs) designs an N'th order
%               lowpass digital EMQF filter with normalized stopband edge
%               frequency Fs. The sampling frequency is assumed to be F0=1.
%               The constant alfa is chosen by designer. For given Fp and Fs
%               (Fp and Fs the boundary frequencies of the specificatin),
%               the permited values alfa are in the range:
%
%   -(1-(tan(pi*Fa)^2)/(1+(tan(pi*Fa)^2)<alfa<-(1-(tan(pi*Fp)^2)/(1+(tan(pi*Fp)^2).
%
%	   The value of alfa approximates 
%              
%               (1-(tan(pi*Fa)(tan(pi*Fp))/(1+(tan(pi*Fa)(tan(pi*Fp))
% 
%               and is adjusted to be represented as a simple combination
%               of powers of two. 
%    
%     In the case of a half-band filter alfa=0         
%
%       emqf_fs returns:
%              * Amplitude response in vector amp, length 500. 
%              * Normalized frequencies (F0=1) in vector f, length 500. 
%              * Square pole magnitudes of the complex poles in vector beta. 
%              * passband ripple Ap (dB)
%              * minimal stopband attenuation As (dB)
%              * passband boundary frequency of EMQF filter Fp
%              * 3 dB attenuation frequency F3db
%
%

disp(' EMQF design')
disp('===============================')
disp([sprintf(' N     = %6u', N)])
disp([sprintf(' alpha = %15.9f', alpha)])
disp([sprintf(' F_s   = %15.9f', Fs)])

F3db = (1/pi)*atan(sqrt((1+alpha)/(1-alpha)));

neven = N-2*fix(N/2);
if neven < 0.5
   disp('------- error -----------------')
   disp('N should be odd number')
   disp('choose new N = 3, 5, 7, 9, 11, ...')
   disp('-------------------------------')
   H=0; w=0; zzd=0; ppd=0; Ap=0; As=0; Fp=0; F3db=0;omega=0;co=0;
   return
end

if F3db >= Fs
   disp([sprintf(' F_3dB = %15.9f', F3db)])
   disp('------- error -----------------')
   disp('Fs should be larger than F3dB')
   disp(['choose new Fs    > ' num2str(F3db) '  or'])
   disp(['choose new alpha < ' num2str(-cos(2*pi*Fs))])
   disp('-------------------------------')
   H=0; w=0; zzd=0; ppd=0; Ap=0; As=0;omega=0;co=0;
   return
end

Fp = (1/pi)*atan(((1+alpha)/(1-alpha))*(1/tan(pi*Fs)));
disp([sprintf(' F_p   = %15.9f', Fp)])
disp([sprintf(' F_3dB = %15.9f', F3db)])

xi = tan(pi*Fs)/tan(pi*Fp);
if xi >= sqrt(2)
   t = .5*(1-(1-1/xi^2)^(1/4))/(1+(1-1/xi^2)^(1/4));
   q = t+2*t^5+15*t^9+150*t^13;
else
   t = .5*(1-1/sqrt(xi))/(1+1/sqrt(xi));
   qp = t+2*t^5+15*t^9+150*t^13;
   q = exp(pi^2/log(qp));
end
L  = (1/4)*sqrt(1/q^N-1);
Ap = 10*log10(1+1/L);
As = 10*log10(1+L);
disp([sprintf(' A_p   =     %15.9g', Ap)])
disp([sprintf(' A_s   = %15.9f', As)])

%[zzd,ppd,co]=ellip(N,Ap,As,2*Fp);
a = xi;
b = 1; 
for ind = 1:(N-1)/2
  x = ellipj( ((2*ind-1)/N + 1)*ellipke(1/a^2), 1/a^2);
  kor = 2*sqrt(a)*tan(pi*Fp)/(1+a*tan(pi*Fp)^2);
  b(ind) = ((a+x^2) - kor*sqrt((1-x^2)*(a^2-x^2)) )/...
           ((a+x^2) + kor*sqrt((1-x^2)*(a^2-x^2)) );
end
a1 = -(1-tan(pi*F3db))/(1+tan(pi*F3db));
beta = sort(b);
disp([sprintf(' alpha1 = %15.9f', a1)])
disp(' beta = ')
disp(beta')

p1 = [1 a1];
for ind = 2:2:(N-1)/2
  p1 = conv(p1,[1 alpha*(1+beta(ind)) beta(ind)]);
end
p2 = 1;
for ind = 1:2:(N-1)/2
  p2 = conv(p2,[1 alpha*(1+beta(ind)) beta(ind)]);
end
pp1=zeros(size(1:M*length(p1)));
pp1([1:M:length(pp1)])=p1;
pp2=zeros(size(1:M*length(p2)));
pp2([1:M:length(pp2)])=p2;
Hab=pp1;
Hai=fliplr(Hab);
Hbb=pp2;
Hbi=fliplr(Hbb);

[Ha,w]=freqz(Hab,Hai,2000);
[Hb,w]=freqz(Hbb,Hbi,2000);
fia=unwrap(angle(Ha));
fib=unwrap(angle(Hb));
psi=(fia-fib)/2;
phase=fia+fib;
amp=cos(psi);
f=w/pi;
plot(f,20*log10(abs(amp)));

%-------------------EOF-----------------------------